@@ -1,4 +1,4 @@
-from .evaluation_details import EvaluationDetails, EvaluationReason
+from .evaluation_details import EvaluationDetails, EvaluationReason, DataSource
 
 
 class _ConfigEvaluation:
@@ -7,6 +7,7 @@ class _ConfigEvaluation:
                  boolean_value=False,
                  json_value=None,
                  rule_id="",
+                 version=None,
                  secondary_exposures=None,
                  allocated_experiment=None,
                  explicit_parameters=None,
@@ -35,9 +36,10 @@ class _ConfigEvaluation:
         self.explicit_parameters = explicit_parameters
         self.is_experiment_group = is_experiment_group is True
         if evaluation_details is None:
-            evaluation_details = EvaluationDetails(0, 0, EvaluationReason.unrecognized)
+            evaluation_details = EvaluationDetails(0, 0, DataSource.UNINITIALIZED, EvaluationReason.none)
         self.evaluation_details = evaluation_details
         self.group_name = group_name
         self.sample_rate = sample_rate
         self.user = user
         self.forward_all_exposures = forward_all_exposures
+        self.version = version
