@@ -3,25 +3,37 @@ from enum import Enum
 
 
 class EvaluationReason(str, Enum):
-    network = "Network"
     local_override = "LocalOverride"
     unrecognized = "Unrecognized"
-    uninitialized = "Uninitialized"
-    bootstrap = "Bootstrap"
-    data_adapter = "DataAdapter"
     unsupported = "Unsupported"
     error = "error"
+    none = "none"
+
+
+class DataSource(str, Enum):
+    DATASTORE = "DataAdapter"
+    BOOTSTRAP = "Bootstrap"
+    NETWORK = "Network"
+    STATSIG_NETWORK = "StatsigNetwork"
+    UNINITIALIZED = "Uninitialized"
 
 
 class EvaluationDetails:
     reason: EvaluationReason
+    source: DataSource
     config_sync_time: int
     init_time: int
     server_time: int
 
     def __init__(self, config_sync_time: int, init_time: int,
-                 reason: EvaluationReason):
+                 source: DataSource, reason: EvaluationReason = EvaluationReason.none):
         self.config_sync_time = config_sync_time
         self.init_time = init_time
         self.reason = reason
+        self.source = source
         self.server_time = round(time.time() * 1000)
+
+    def detailed_reason(self):
+        if self.reason == EvaluationReason.none:
+            return f"{self.source.value}"
+        return f"{self.source.value}:{self.reason.value}"
