@@ -3,7 +3,7 @@ import re
 import time
 from datetime import datetime
 from hashlib import sha256
-from typing import Dict
+from typing import Dict, Optional
 
 from ip3country import CountryLookup
 from ua_parser import user_agent_parser
@@ -11,7 +11,7 @@ from ua_parser import user_agent_parser
 from . import globals
 from .client_initialize_formatter import ClientInitializeResponseFormatter
 from .config_evaluation import _ConfigEvaluation
-from .evaluation_details import EvaluationDetails, EvaluationReason
+from .evaluation_details import EvaluationDetails, EvaluationReason, DataSource
 from .spec_store import _SpecStore
 from .statsig_user import StatsigUser
 from .utils import HashingAlgorithm, sha256_hash
@@ -21,11 +21,14 @@ class _Evaluator:
     def __init__(self, spec_store: _SpecStore):
         self._spec_store = spec_store
 
-        self._country_lookup = CountryLookup()
+        self._country_lookup: Optional[CountryLookup] = None
         self._gate_overrides: Dict[str, dict] = {}
         self._config_overrides: Dict[str, dict] = {}
         self._layer_overrides: Dict[str, dict] = {}
 
+    def initialize(self):
+        self._country_lookup = CountryLookup()
+
     def override_gate(self, gate, value, user_id=None):
         gate_overrides = self._gate_overrides.get(gate)
         if gate_overrides is None:
@@ -105,20 +108,23 @@ class _Evaluator:
             .get_formatted_response(self.__eval_config, user, self._spec_store, self, hash, client_sdk_key,
                                     include_local_override)
 
-    def _create_evaluation_details(self, reason: EvaluationReason):
-        if reason == EvaluationReason.uninitialized:
-            return EvaluationDetails(0, 0, reason)
+    def _create_evaluation_details(self,
+                                   reason: EvaluationReason = EvaluationReason.none,
+                                   source: Optional[DataSource] = None):
+        if source is None:
+            source = self._spec_store.init_source
+        if source == DataSource.UNINITIALIZED:
+            return EvaluationDetails(0, 0, source, reason)
 
         return EvaluationDetails(
-            self._spec_store.last_update_time(), self._spec_store.initial_update_time, reason)
+            self._spec_store.last_update_time(), self._spec_store.initial_update_time, source, reason)
 
     def __lookup_gate_override(self, user, gate):
         gate_overrides = self._gate_overrides.get(gate)
         if gate_overrides is None:
             return None
 
-        eval_details = self._create_evaluation_details(
-            EvaluationReason.local_override)
+        eval_details = self._create_evaluation_details(EvaluationReason.local_override)
         override = self.__lookup_override(gate_overrides, user)
         if override is not None:
             return _ConfigEvaluation(boolean_value=override, rule_id="override",
@@ -139,8 +145,7 @@ class _Evaluator:
         if config_overrides is None:
             return None
 
-        eval_details = self._create_evaluation_details(
-            EvaluationReason.local_override)
+        eval_details = self._create_evaluation_details(EvaluationReason.local_override)
         override = self.__lookup_override(config_overrides, user)
         if override is not None:
             return _ConfigEvaluation(json_value=override, rule_id="override",
@@ -160,8 +165,7 @@ class _Evaluator:
         if layer_overrides is None:
             return None
 
-        eval_details = self._create_evaluation_details(
-            EvaluationReason.local_override)
+        eval_details = self._create_evaluation_details(EvaluationReason.local_override)
         override = self.__lookup_override(layer_overrides, user)
         if override is not None:
             return _ConfigEvaluation(json_value=override, rule_id="override",
@@ -185,27 +189,21 @@ class _Evaluator:
     def unsupported_or_unrecognized(self, config_name):
         if config_name in self._spec_store.unsupported_configs:
             return _ConfigEvaluation(
-                evaluation_details=self._create_evaluation_details(
-                    EvaluationReason.unsupported))
+                evaluation_details=self._create_evaluation_details(EvaluationReason.unsupported))
         return _ConfigEvaluation(
-            evaluation_details=self._create_evaluation_details(
-                EvaluationReason.unrecognized))
+            evaluation_details=self._create_evaluation_details(EvaluationReason.unrecognized))
 
     def check_gate(self, user, gate, end_result=None, is_nested=False):
         override = self.__lookup_gate_override(user, gate)
         if override is not None:
             return override
 
-        if self._spec_store.init_reason == EvaluationReason.uninitialized:
-            return _ConfigEvaluation(
-                evaluation_details=self._create_evaluation_details(
-                    EvaluationReason.uninitialized))
         eval_gate = self._spec_store.get_gate(gate)
         if eval_gate is None:
             globals.logger.debug(f"Gate {gate} not found in the store. Are you sure the gate name is correct?")
             return self.unsupported_or_unrecognized(gate)
         if end_result is None:
-            end_result = _ConfigEvaluation()
+            end_result = _ConfigEvaluation(version=eval_gate.get("version", None))
         self.__eval_config(user, eval_gate, end_result, is_nested)
         return end_result
 
@@ -214,18 +212,13 @@ class _Evaluator:
         if override is not None:
             return override
 
-        if self._spec_store.init_reason == EvaluationReason.uninitialized:
-            return _ConfigEvaluation(
-                evaluation_details=self._create_evaluation_details(
-                    EvaluationReason.uninitialized))
-
         eval_config = self._spec_store.get_config(config_name)
         if eval_config is None:
             globals.logger.debug(
                 f"{'Config' if is_exp else 'Experiment'} {config_name} not found in the store. Are you sure the config name is correct?"
             )
             return self.unsupported_or_unrecognized(config_name)
-        result = _ConfigEvaluation()
+        result = _ConfigEvaluation(version=eval_config.get("version", None))
         self.__eval_config(user, eval_config, result)
         return result
 
@@ -234,33 +227,25 @@ class _Evaluator:
         if override is not None:
             return override
 
-        if self._spec_store.init_reason == EvaluationReason.uninitialized:
-            return _ConfigEvaluation(
-                evaluation_details=self._create_evaluation_details(
-                    EvaluationReason.uninitialized))
-
         eval_layer = self._spec_store.get_layer(layer_name)
         if eval_layer is None:
             globals.logger.debug(f"Layer {layer_name} not found in the store. Are you sure the layer name is correct?")
             return self.unsupported_or_unrecognized(layer_name)
-        result = _ConfigEvaluation()
+        result = _ConfigEvaluation(version=eval_layer.get("version", None))
         self.__eval_config(user, eval_layer, result)
         return result
 
     def __eval_config(self, user, config, end_result, is_nested=False):
         if config is None:
-            end_result.evaluation_details = self._create_evaluation_details(
-                EvaluationReason.unrecognized)
+            end_result.evaluation_details = self._create_evaluation_details(EvaluationReason.unrecognized)
             return
         try:
             self.__evaluate(user, config, end_result, is_nested)
-            end_result.evaluation_details = self._create_evaluation_details(
-                self._spec_store.init_reason)
+            end_result.evaluation_details = self._create_evaluation_details()
         except RecursionError:
             raise
         except Exception:
-            end_result.evaluation_details = self._create_evaluation_details(
-                EvaluationReason.error)
+            end_result.evaluation_details = self._create_evaluation_details(EvaluationReason.error)
             end_result.rule_id = "error"
 
     def __check_id_in_list(self, id, list_name):
@@ -383,11 +368,15 @@ class _Evaluator:
             if value is None:
                 ip = self.__get_from_user(user, "ip")
                 if ip is not None and field == "country":
+                    if not self._country_lookup:
+                        self._country_lookup = CountryLookup()
                     value = self._country_lookup.lookupStr(ip)
             if value is None:
                 return False
         elif type == "UA_BASED":
-            value = self.__get_from_user_agent(user, field)
+            value = self.__get_from_user(user, field)
+            if value is None:
+                value = self.__get_from_user_agent(user, field)
         elif type == "USER_FIELD":
             value = self.__get_from_user(user, field)
         elif type == "CURRENT_TIME":
