@@ -10,6 +10,7 @@ import grpc
 
 from . import globals
 from .diagnostics import Marker, Diagnostics
+from .evaluation_details import DataSource
 from .grpc.generated.statsig_forward_proxy_pb2 import (ConfigSpecRequest)  # pylint: disable=no-name-in-module
 from .grpc.generated.statsig_forward_proxy_pb2_grpc import StatsigForwardProxyStub
 from .interface_network import (
@@ -19,6 +20,7 @@ from .interface_network import (
     IStreamingListeners,
     IStatsigWebhookWorker,
 )
+from .statsig_context import InitContext
 from .statsig_error_boundary import _StatsigErrorBoundary
 from .statsig_errors import StatsigNameError
 from .statsig_options import ProxyConfig, StatsigOptions, AuthenticationMode
@@ -62,7 +64,9 @@ class GRPCWebsocketWorker(IStatsigNetworkWorker, IStatsigWebhookWorker):
             error_boundary: _StatsigErrorBoundary,
             diagnostics: Diagnostics,
             shutdown_event,
+            context: InitContext
     ):
+        self.context = context
         self._diagnostics = diagnostics
         self.initialized = False
         self.sdk_key = sdk_key
@@ -202,6 +206,7 @@ class GRPCWebsocketWorker(IStatsigNetworkWorker, IStatsigWebhookWorker):
             log_on_exception: Optional[bool] = False,
             init_timeout: Optional[int] = None,
     ):
+        self.context.source_api = self.proxy_config.proxy_address
         self._diagnostics.add_marker(
             Marker().download_config_specs().network_request().start()
         )
@@ -224,7 +229,7 @@ class GRPCWebsocketWorker(IStatsigNetworkWorker, IStatsigWebhookWorker):
                     }
                 )
             )
-            on_complete(json.loads(dcs_data.spec), None)
+            on_complete(DataSource.NETWORK, json.loads(dcs_data.spec), None)
         except Exception as e:
             self.error_boundary.log_exception("grpcWebSocket:initialize", e)
             self._diagnostics.add_marker(
@@ -239,7 +244,7 @@ class GRPCWebsocketWorker(IStatsigNetworkWorker, IStatsigWebhookWorker):
                     }
                 )
             )
-            on_complete(None, e)
+            on_complete(DataSource.NETWORK, None, e)
 
     def get_id_lists(
             self,
