@@ -2,17 +2,18 @@ import json
 import threading
 from concurrent.futures import wait, ThreadPoolExecutor
 from enum import Enum
-from typing import List, Optional, Dict, Set, Tuple
+from typing import List, Optional, Dict, Set, Tuple, Union
 
 from . import globals
 from .constants import Const
-from .diagnostics import Context, Diagnostics, Marker
-from .evaluation_details import EvaluationReason
+from .diagnostics import Diagnostics, Marker
+from .evaluation_details import EvaluationReason, DataSource
 from .sdk_configs import _SDK_Configs
 from .spec_updater import SpecUpdater
+from .statsig_context import InitContext
 from .statsig_error_boundary import _StatsigErrorBoundary
 from .statsig_network import _StatsigNetwork
-from .statsig_options import DataSource, StatsigOptions
+from .statsig_options import StatsigOptions
 from .utils import djb2_hash
 
 
@@ -35,10 +36,11 @@ class _SpecStore:
             shutdown_event: threading.Event,
             sdk_key: str,
             diagnostics: Diagnostics,
+            context: InitContext
     ):
         self.initial_update_time = 0
-        self.init_reason = EvaluationReason.uninitialized
-        self.init_source = DataSource.NETWORK
+        self.init_reason = EvaluationReason.none
+        self.init_source = DataSource.UNINITIALIZED
         self._options = options
         self._statsig_metadata = statsig_metadata
         self._error_boundary = error_boundary
@@ -52,9 +54,11 @@ class _SpecStore:
         self._experiment_to_layer: Dict[str, str] = {}
         self._sdk_keys_to_app_ids: Dict[str, str] = {}
         self._hashed_sdk_keys_to_app_ids: Dict[str, str] = {}
+        self._default_environment: Union[None, str] = None
 
         self._id_lists: Dict[str, dict] = {}
         self.unsupported_configs: Set[str] = set()
+        self.context = context
 
         self.spec_updater = SpecUpdater(
             network,
@@ -65,6 +69,7 @@ class _SpecStore:
             error_boundary,
             statsig_metadata,
             shutdown_event,
+            context
         )
 
         self.spec_updater.register_process_network_id_lists_listener(
@@ -83,7 +88,6 @@ class _SpecStore:
         )
 
         self.spec_updater.download_id_lists(for_initialize=True)
-
         self.spec_updater.start_background_threads()
         self.spec_updater.initialized = True
 
@@ -136,26 +140,35 @@ class _SpecStore:
             return target_app_id
         return self._sdk_keys_to_app_ids.get(sdk_key)
 
+    def get_default_environment(self):
+        return self._default_environment
+
     def _initialize_specs(self):
         initialize_strategies = self._get_initialize_strategy()
         for strategy in initialize_strategies:
             self.spec_updater.get_config_spec(strategy, True)
-            if self.init_reason is EvaluationReason.bootstrap or self.last_update_time() != 0:
+            if self.init_source is DataSource.BOOTSTRAP or self.last_update_time() != 0:
                 self.init_source = strategy
                 break
 
-    def _process_specs(self, specs_json, reason: EvaluationReason) -> Tuple[bool, bool]:  # has update, parse success
+    def _process_specs(self, specs_json, source: DataSource) -> Tuple[bool, bool]:  # has update, parse success
         self._log_process("Processing specs...")
-        if specs_json.get("has_updates", False) is False:
-            globals.logger.debug("Received update: %s", "No Update")
+        prev_lcut = self.spec_updater.last_update_time
+        if specs_json.get("has_updates") is not None and not specs_json.get("has_updates"):  # 204 no update
+            globals.logger.log_config_sync_update(self.spec_updater.initialized, False,
+                                                  self.spec_updater.last_update_time,
+                                                  prev_lcut,
+                                                  self.context.source,
+                                                  self.context.source_api)
             return False, True
         if not self.spec_updater.is_specs_json_valid(specs_json):
-            self._log_process("Failed to process specs")
+            self._log_process("Failed to process specs, invalid spec json")  # invalid json
             return False, False
-        if specs_json.get("time", 0) < self.last_update_time():
+        if specs_json.get("time", 0) < self.last_update_time():  # outdated lcut
+            self._log_process("Failed to process specs, lcut is older than current lcut")
             return False, False
-        copy = json.dumps(specs_json)
         if callable(self._options.rules_updated_callback):
+            copy = json.dumps(specs_json)
             self._options.rules_updated_callback(copy)
 
         def get_parsed_specs(key: str):
@@ -229,9 +242,15 @@ class _SpecStore:
         self._layers = new_layers
         self._experiment_to_layer = new_experiment_to_layer
         self.spec_updater.last_update_time = specs_json.get("time", 0)
-        self.init_reason = reason
-        globals.logger.debug("Received update: %s", self.spec_updater.last_update_time)
-
+        self.init_source = source
+        self.context.source = source
+        self._default_environment = specs_json.get("default_environment", None)
+        if self.spec_updater.last_update_time > prev_lcut:
+            globals.logger.log_config_sync_update(self.spec_updater.initialized, True,
+                                                  self.spec_updater.last_update_time,
+                                                  prev_lcut,
+                                                  self.context.source,
+                                                  self.context.source_api)
         flags = specs_json.get("sdk_flags", {})
         _SDK_Configs.set_flags(flags)
         configs = specs_json.get("sdk_configs", {})
@@ -343,13 +362,6 @@ class _SpecStore:
             process = "Initialize" if not self.spec_updater.initialized else "Config Sync"
         globals.logger.log_process(process, msg)
 
-    def _get_current_context(self):
-        return (
-            Context.INITIALIZE
-            if not self.spec_updater.initialized
-            else Context.CONFIG_SYNC
-        )
-
     def _get_initialize_strategy(self) -> List[DataSource]:
         try:
             if self._options.initialize_sources is not None:
@@ -370,6 +382,6 @@ class _SpecStore:
             return strategies
         except Exception:
             globals.logger.warning(
-                "Failed to get initialization sources, fallling back to always sync from statsig network "
+                "Failed to get initialization sources, falling back to always sync from statsig network "
             )
             return [DataSource.STATSIG_NETWORK]
