@@ -2,9 +2,11 @@ from enum import Enum
 from typing import List, Optional, Union, Callable, Dict, Any
 
 from .dynamic_config import DynamicConfig
+from .evaluation_details import DataSource
 from .feature_gate import FeatureGate
 from .interface_data_store import IDataStore
 from .interface_network import NetworkProtocol, NetworkEndpoint
+from .interface_observability_client import ObservabilityClient
 from .layer import Layer
 from .output_logger import OutputLogger, LogLevel
 from .statsig_environment_tier import StatsigEnvironmentTier
@@ -62,13 +64,6 @@ class ProxyConfig:
         self.tls_ca_cert_path = tls_ca_cert_path
 
 
-class DataSource(str, Enum):
-    DATASTORE = "datastore"
-    BOOTSTRAP = "bootstrap"
-    NETWORK = "network"
-    STATSIG_NETWORK = "statsig_network"
-
-
 DEFAULT_PROXY_CONFIG = {
     NetworkEndpoint.ALL: ProxyConfig(
         proxy_address=STATSIG_API, protocol=NetworkProtocol.HTTP
@@ -113,6 +108,11 @@ class StatsigOptions:
             initialize_sources: Optional[List[DataSource]] = None,
             config_sync_sources: Optional[List[DataSource]] = None,
             output_logger_level: Optional[LogLevel] = LogLevel.WARNING,
+            overall_init_timeout: Optional[float] = None,
+            observability_client: Optional[ObservabilityClient] = None,
+            sdk_error_callback: Optional[Callable[[str, Exception], None]] = None,
+            events_flushed_callback: Optional[
+                Callable[[bool, List[Dict], Optional[int], Optional[Exception]], None]] = None,
     ):
         self.data_store = data_store
         self._environment: Union[None, dict] = None
@@ -158,6 +158,10 @@ class StatsigOptions:
         self.initialize_sources = initialize_sources
         self.config_sync_sources = config_sync_sources
         self.output_logger_level = output_logger_level
+        self.overall_init_timeout = overall_init_timeout
+        self.observability_client = observability_client
+        self.sdk_error_callback = sdk_error_callback
+        self.events_flushed_callback = events_flushed_callback
         self._logging_copy: Dict[str, Any] = {}
         self._set_logging_copy()
         self._attributes_changed = False
@@ -204,7 +208,7 @@ class StatsigOptions:
             logging_copy["init_timeout"] = self.init_timeout
         if self.timeout:
             logging_copy["timeout"] = self.timeout
-        if (self.rulesets_sync_interval) != DEFAULT_RULESET_SYNC_INTERVAL:
+        if self.rulesets_sync_interval != DEFAULT_RULESET_SYNC_INTERVAL:
             logging_copy["rulesets_sync_interval"] = self.rulesets_sync_interval
         if self.idlists_sync_interval != DEFAULT_IDLIST_SYNC_INTERVAL:
             logging_copy["idlists_sync_interval"] = self.idlists_sync_interval
@@ -220,9 +224,25 @@ class StatsigOptions:
             logging_copy["logging_interval"] = self.logging_interval
         if self.disable_diagnostics:
             logging_copy["disable_diagnostics"] = self.disable_diagnostics
+        if self.disable_all_logging:
+            logging_copy["disable_all_logging"] = self.disable_all_logging
         if self.event_queue_size != DEFAULT_EVENT_QUEUE_SIZE:
             logging_copy["event_queue_size"] = self.event_queue_size
         if self.retry_queue_size != DEFAULT_RETRY_QUEUE_SIZE:
             logging_copy["retry_queue_size"] = self.retry_queue_size
+        if self.overall_init_timeout is not None:
+            logging_copy["overall_init_timeout"] = self.overall_init_timeout
+        if self.observability_client is not None:
+            logging_copy["observability_client"] = "SET"
+        if self.fallback_to_statsig_api:
+            logging_copy["fallback_to_statsig_api"] = self.fallback_to_statsig_api
+        if self.out_of_sync_threshold_in_s:
+            logging_copy["out_of_sync_threshold_in_s"] = self.out_of_sync_threshold_in_s
+        if self.initialize_sources:
+            logging_copy["initialize_sources"] = "SET"
+        if self.config_sync_sources:
+            logging_copy["config_sync_sources"] = "SET"
+        if self.sdk_error_callback:
+            logging_copy["sdk_error_callback"] = "SET"
         self._logging_copy = logging_copy
         self._attributes_changed = False
