@@ -1,27 +1,30 @@
 import dataclasses
 import threading
-from typing import Optional, Union, Tuple
+import time
+from concurrent.futures import ThreadPoolExecutor
+from typing import Optional, Union
 
 from . import globals
 from .config_evaluation import _ConfigEvaluation
 from .diagnostics import Context, Diagnostics, Marker
 from .dynamic_config import DynamicConfig
+from .evaluation_details import DataSource
 from .evaluator import _Evaluator
 from .feature_gate import FeatureGate
+from .initialize_details import InitializeDetails
 from .layer import Layer
-from .sdk_configs import _SDK_Configs
-from .spec_store import _SpecStore, EntityType
+from .spec_store import _SpecStore
+from .statsig_context import InitContext
 from .statsig_error_boundary import _StatsigErrorBoundary
-from .statsig_errors import StatsigNameError, StatsigRuntimeError, StatsigValueError
+from .statsig_errors import StatsigNameError, StatsigRuntimeError, StatsigValueError, StatsigTimeoutError
 from .statsig_event import StatsigEvent
 from .statsig_logger import _StatsigLogger
 from .statsig_metadata import _StatsigMetadata
 from .statsig_network import _StatsigNetwork
 from .statsig_options import StatsigOptions
 from .statsig_user import StatsigUser
-from .ttl_set import TTLSet
-from .utils import HashingAlgorithm, compute_dedupe_key_for_gate, is_hash_in_sampling_rate, \
-    compute_dedupe_key_for_config, compute_dedupe_key_for_layer
+from .utils import HashingAlgorithm
+from .version import __version__
 
 RULESETS_SYNC_INTERVAL = 10
 IDLISTS_SYNC_INTERVAL = 60
@@ -41,6 +44,7 @@ class StatsigServer:
     _evaluator: _Evaluator
 
     def __init__(self) -> None:
+        self._executor = ThreadPoolExecutor(max_workers=2)
         self._initialized = False
 
         self._errorBoundary = _StatsigErrorBoundary()
@@ -48,26 +52,42 @@ class StatsigServer:
     def is_initialized(self):
         return self._initialized
 
-    def initialize(self, sdkKey: str, options: Optional[StatsigOptions] = None):
+    def initialize(self, sdkKey: str, options: Optional[StatsigOptions] = None) -> InitializeDetails:
+        if options is None or not isinstance(options, StatsigOptions):
+            options = StatsigOptions()
+
+        globals.init_logger(options)
+
         if self._initialized:
             globals.logger.info("Statsig is already initialized.")
-            return
+            return InitializeDetails(0, self.get_init_source(), True, self.is_store_populated(), None)
 
         if sdkKey is None or not sdkKey.startswith("secret-"):
             raise StatsigValueError(
                 "Invalid key provided. You must use a Server Secret Key from the Statsig console."
             )
 
-        self._initialize_impl(sdkKey, options)
+        environment_tier = options.get_sdk_environment_tier() or 'unknown'
+        globals.logger.info(
+            f"Initializing Statsig SDK (v{__version__}) instance. "
+            f"Current environment tier: {environment_tier}."
+        )
+
+        init_details = self._initialize_impl(sdkKey, options)
+        globals.logger.log_post_init(options, init_details)
+
+        return init_details
 
     def _initialize_impl(self, sdk_key: str, options: Optional[StatsigOptions]):
         threw_error = False
+        init_context = InitContext()
         try:
             diagnostics = Diagnostics()
             diagnostics.add_marker(Marker().overall().start())
 
             self._errorBoundary.set_api_key(sdk_key)
             self._errorBoundary.set_diagnostics(diagnostics)
+            self._errorBoundary.set_init_context(init_context)
             if options is None:
                 options = StatsigOptions()
             self._options = options
@@ -77,7 +97,8 @@ class StatsigServer:
                 self._options, self.__statsig_metadata
             )
             self._network = _StatsigNetwork(
-                sdk_key, self._options, self.__statsig_metadata, self._errorBoundary, diagnostics, self.__shutdown_event
+                sdk_key, self._options, self.__statsig_metadata, self._errorBoundary, diagnostics,
+                self.__shutdown_event, init_context
             )
             self._logger = _StatsigLogger(
                 self._network,
@@ -98,19 +119,34 @@ class StatsigServer:
                 self._errorBoundary,
                 self.__shutdown_event,
                 sdk_key,
-                diagnostics
+                diagnostics,
+                init_context
             )
 
             self._evaluator = _Evaluator(self._spec_store)
-            self._sampling_key_set = TTLSet(self.__shutdown_event)
 
-            self._spec_store.initialize()
-            self._initialized = True
+            init_timeout = options.overall_init_timeout
+            if init_timeout is not None:
+                future = self._executor.submit(self._initialize_instance)
+                try:
+                    future.result(timeout=init_timeout)
+                except Exception as e:
+                    raise StatsigTimeoutError() from e
+            else:
+                self._initialize_instance()
 
         except (StatsigValueError, StatsigNameError, StatsigRuntimeError) as e:
             threw_error = True
             raise e
 
+        except StatsigTimeoutError:
+            globals.logger.warning(
+                "Statsig SDK instance initialization timed out. Initializing store in the background.")
+            self._executor.submit(self._initialize_instance)
+            threw_error = True
+            init_context.timed_out = True
+            self._initialized = True
+
         except Exception as e:
             threw_error = True
             self._errorBoundary.log_exception("initialize", e)
@@ -118,12 +154,30 @@ class StatsigServer:
         finally:
             diagnostics.add_marker(Marker().overall().end({"success": not threw_error}))
             diagnostics.log_diagnostics(Context.INITIALIZE)
+            init_context.source = self.get_init_source()
+            init_context.store_populated = self.is_store_populated()
+            init_context.success = not threw_error
+
+        return InitializeDetails(int(time.time() * 1000) - init_context.start_time, init_context.source,
+                                 init_context.success, init_context.store_populated, init_context.error,
+                                 init_context.source_api, init_context.timed_out)
+
+    def _initialize_instance(self):
+        self._evaluator.initialize()
+        self._spec_store.initialize()
+        self._initialized = True
 
     def is_store_populated(self):
-        return self._spec_store.last_update_time() > 0
+        try:
+            return self._spec_store.last_update_time() > 0
+        except Exception:
+            return False
 
     def get_init_source(self):
-        return self._spec_store.init_source
+        try:
+            return self._spec_store.init_source
+        except Exception:
+            return DataSource.UNINITIALIZED
 
     def get_feature_gate(self, user: StatsigUser, gate_name: str, log_exposure=True):
         def task():
@@ -167,10 +221,7 @@ class StatsigServer:
         self._logger.log_gate_exposure(
             user,
             gate_name,
-            result.boolean_value,
-            result.rule_id,
-            result.secondary_exposures,
-            result.evaluation_details,
+            result,
             is_manual_exposure=True,
         )
 
@@ -190,7 +241,9 @@ class StatsigServer:
                 result.user,
                 group_name=result.group_name,
                 evaluation_details=result.evaluation_details,
-                secondary_exposures=result.secondary_exposures
+                secondary_exposures=result.secondary_exposures,
+                passed_rule=result.boolean_value,
+                version=result.version,
             )
             self.safe_eval_callback(dynamicConfig)
             return dynamicConfig
@@ -208,9 +261,7 @@ class StatsigServer:
         self._logger.log_config_exposure(
             user,
             config_name,
-            result.rule_id,
-            result.secondary_exposures,
-            result.evaluation_details,
+            result,
             is_manual_exposure=True,
         )
 
@@ -219,9 +270,15 @@ class StatsigServer:
         self._logger.log_config_exposure(
             user,
             config_eval.name,
-            config_eval.rule_id,
-            config_eval.secondary_exposures,
-            config_eval.evaluation_details,
+            _ConfigEvaluation(
+                config_eval.passed_rule,
+                config_eval.value,
+                config_eval.rule_id,
+                config_eval.version,
+                config_eval.secondary_exposures,
+                evaluation_details=config_eval.evaluation_details,
+
+            ),
             is_manual_exposure=True,
         )
 
@@ -243,6 +300,8 @@ class StatsigServer:
                 group_name=result.group_name,
                 evaluation_details=result.evaluation_details,
                 secondary_exposures=result.secondary_exposures,
+                passed_rule=result.boolean_value,
+                version=result.version,
             )
             self.safe_eval_callback(dynamicConfig)
             return dynamicConfig
@@ -260,9 +319,7 @@ class StatsigServer:
         self._logger.log_config_exposure(
             user,
             experiment_name,
-            result.rule_id,
-            result.secondary_exposures,
-            result.evaluation_details,
+            result,
             is_manual_exposure=True,
         )
 
@@ -278,14 +335,10 @@ class StatsigServer:
             result = self._evaluator.get_layer(normal_user, layer_name)
 
             def log_func(layer: Layer, parameter_name: str):
-                should_log, logged_sampling_rate, shadow_logged = self.__determine_sampling(
-                    EntityType.LAYER, layer_name, result, user, parameter_name)
 
-                if log_exposure and should_log:
+                if log_exposure:
                     self._logger.log_layer_exposure(
-                        normal_user, layer, parameter_name, result, sampling_rate=logged_sampling_rate,
-                        shadow_logged=shadow_logged,
-                        sampling_mode=_SDK_Configs.get_config_str_value("sampling_mode")
+                        normal_user, layer, parameter_name, result,
                     )
 
             layer = Layer._create(
@@ -347,11 +400,13 @@ class StatsigServer:
 
     def shutdown(self):
         def task():
+            globals.logger.info("Shutting down Statsig SDK instance.")
             self.__shutdown_event.set()
             self._logger.shutdown()
             self._spec_store.shutdown()
             self._network.shutdown()
             self._errorBoundary.shutdown()
+            globals.logger.shutdown()
             self._initialized = False
 
         self._errorBoundary.swallow("shutdown", task)
@@ -495,20 +550,12 @@ class StatsigServer:
     def __check_gate(self, user: StatsigUser, gate_name: str, log_exposure=True):
         user = self.__normalize_user(user)
         result = self._evaluator.check_gate(user, gate_name)
-        should_log, logged_sampling_rate, shadow_logged = self.__determine_sampling(EntityType.GATE, gate_name, result,
-                                                                                    user)
 
-        if log_exposure and should_log:
+        if log_exposure:
             self._logger.log_gate_exposure(
                 user,
                 gate_name,
-                result.boolean_value,
-                result.rule_id,
-                result.secondary_exposures,
-                result.evaluation_details,
-                sampling_rate=logged_sampling_rate,
-                shadow_logged=shadow_logged,
-                sampling_mode=_SDK_Configs.get_config_str_value("sampling_mode")
+                result,
             )
         return result
 
@@ -517,80 +564,22 @@ class StatsigServer:
 
         result = self._evaluator.get_config(user, config_name)
         result.user = user
-        should_log, logged_sampling_rate, shadow_logged = self.__determine_sampling(EntityType.CONFIG, config_name,
-                                                                                    result, user)
 
-        if log_exposure and should_log:
+        if log_exposure:
             self._logger.log_config_exposure(
                 user,
                 config_name,
-                result.rule_id,
-                result.secondary_exposures,
-                result.evaluation_details,
-                sampling_rate=logged_sampling_rate,
-                shadow_logged=shadow_logged,
-                sampling_mode=_SDK_Configs.get_config_str_value("sampling_mode")
+                result
             )
         return result
 
-    def __determine_sampling(self, type: EntityType, name: str, result: _ConfigEvaluation, user: StatsigUser,
-                             param_name="") -> Tuple[
-        bool, Optional[int], Optional[str]]:  # should_log, logged_sampling_rate, shadow_logged
-        try:
-            shadow_should_log, logged_sampling_rate = True, None
-            env = self._options.get_sdk_environment_tier()
-            sampling_mode = _SDK_Configs.get_config_str_value("sampling_mode")
-            special_case_sampling_rate = _SDK_Configs.get_config_int_value("special_case_sampling_rate")
-            special_case_rules = ["disabled", "default", ""]
-
-            if sampling_mode is None or sampling_mode == "none" or env != "production":
-                return True, None, None
-
-            if result.forward_all_exposures:
-                return True, None, None
-
-            samplingSetKey = f"{name}_{result.rule_id}"
-            if not self._sampling_key_set.contains(samplingSetKey):
-                self._sampling_key_set.add(samplingSetKey)
-                return True, None, None
-
-            should_sample = result.sample_rate is not None or result.rule_id in special_case_rules
-            if not should_sample:
-                return True, None, None
-
-            exposure_key = ""
-            if type == EntityType.GATE:
-                exposure_key = compute_dedupe_key_for_gate(name, result.rule_id, result.boolean_value,
-                                                           user.user_id, user.custom_ids)
-            elif type == EntityType.CONFIG:
-                exposure_key = compute_dedupe_key_for_config(name, result.rule_id, user.user_id, user.custom_ids)
-            elif type == EntityType.LAYER:
-                exposure_key = compute_dedupe_key_for_layer(name, result.allocated_experiment, param_name,
-                                                            result.rule_id,
-                                                            user.user_id, user.custom_ids)
-
-            if result.sample_rate is not None:
-                shadow_should_log = is_hash_in_sampling_rate(exposure_key, result.sample_rate)
-                logged_sampling_rate = result.sample_rate
-            elif result.rule_id in special_case_rules and special_case_sampling_rate is not None:
-                shadow_should_log = is_hash_in_sampling_rate(exposure_key, special_case_sampling_rate)
-                logged_sampling_rate = special_case_sampling_rate
-
-            shadow_logged = None if result.sample_rate is None else "logged" if shadow_should_log else "dropped"
-            if sampling_mode == "on":
-                return shadow_should_log, logged_sampling_rate, shadow_logged
-            if sampling_mode == "shadow":
-                return True, logged_sampling_rate, shadow_logged
-
-            return True, None, None
-        except Exception as e:
-            self._errorBoundary.log_exception("__determine_sampling", e, log_mode="debug")
-            return True, None, None
-
     def __normalize_user(self, user):
         userCopy = dataclasses.replace(user)
+        default_env = self._spec_store.get_default_environment()
         if self._options is not None and self._options._environment is not None:
             userCopy._statsig_environment = self._options._environment
+        if userCopy._statsig_environment is None and default_env is not None:
+            userCopy._statsig_environment = {"tier": default_env}
         return userCopy
 
     def _sync(self, sync_func, interval):
