@@ -1,10 +1,15 @@
 from dataclasses import dataclass
-from typing import Optional
+from typing import Optional, Union, Sequence, Mapping, Dict # pylint: disable=unused-import
+
+from typing_extensions import TypeAliasType
 
 from .statsig_environment_tier import StatsigEnvironmentTier
 from .statsig_errors import StatsigValueError
 from .utils import str_or_none, to_raw_dict_or_none, djb2_hash_for_dict
 
+JSONPrimitive = Union[str, int, float, bool, None]
+JSONValue = TypeAliasType("JSONValue", 'Union[JSONPrimitive, Sequence["JSONValue"], Mapping[str, "JSONValue"]]')
+
 
 @dataclass
 class StatsigUser:
@@ -21,10 +26,10 @@ class StatsigUser:
     country: Optional[str] = None
     locale: Optional[str] = None
     app_version: Optional[str] = None
-    custom: Optional[dict] = None
-    private_attributes: Optional[dict] = None
-    custom_ids: Optional[dict] = None
-    _statsig_environment: Optional[dict] = None
+    custom: Optional[Mapping[str, JSONValue]] = None
+    private_attributes: Optional[Mapping[str, JSONValue]] = None
+    custom_ids: Optional[Dict[str, str]] = None
+    _statsig_environment: Optional[Mapping[str, JSONValue]] = None
 
     def __post_init__(self):
         # ensure there is a user id or at least a custom ID, empty dict
